@@ -30,6 +30,12 @@ enum {NAV_NW, NAV_NE, NAV_SE, NAV_SW};
 enum {DIRM_LAST, DIRM_FIXED};
 enum {RAR_FILE, ZIP_FILE};
 
+enum {	ACC_FUNCTION,
+	ACC_MODS,
+	ACC_ACCEL,
+	ACC_COLS
+};
+
 typedef struct {
   // Preferences that are saved
   char	lastbook[200],
@@ -58,3 +64,7 @@ void		save_config(void);
 void		load_config(void);
 void		show_config(GtkWidget *ref);
 void		parse_config(GtkWidget *ref);
+
+void		conf_accel_tview_cursor_changed(void);
+void		conf_accel_tview_realize(GtkTreeView *tview);
+void		set_kay_accelerators(void);
