@@ -27,6 +27,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
+#include <unistd.h>
+#include <sys/types.h>	/* open */
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <sys/wait.h>	/* wait */
 
 #include "support.h"
 #include "interface.h"
@@ -51,6 +57,7 @@ char		*extens[] = {
 			"\\.JPG\\|\\.JPEG\\|\\.PNG\\|\\.GIF";
 int		 nr_extensions = 8,
 		 ext_nr = 0;
+gboolean	 file_list_expanded = FALSE;
 
 
 void
@@ -138,22 +145,18 @@ void
 start_show(void)
 {
   GtkWidget *w;
-  char *bff, *p, **names;
+  char *bff, *p = NULL, **names;
+  int pfd[2];			/* pipe Mamoru Tasaka */
+  int pid_i, pid_j;		/* two children Mamoru Tasaka */
   gboolean first = TRUE;
-  int s, t, bffbeg;
+  int s, t, bffbeg = 0;
 
   if (debug) printf("%s\n", pref.lastbook);
 
   switch (pref.booktype = file_type_of(pref.lastbook)) {
     case ZIP_FILE:
-      bff = g_strdup_printf("unzip -l \"%s\" | grep \"%s\" > %s",
-		pref.lastbook, all_extensions, tmpf);
-      if (debug) printf("ZIP command: %s\n", bff);
       break;
     case RAR_FILE:
-      bff = g_strdup_printf("unrar v \"%s\" | grep \"%s\" > %s",
-		pref.lastbook, all_extensions, tmpf);
-      if (debug) printf("RAR command: %s\n", bff);
       break;
     default:			// Patch from Ilja Pyykkonen 2005/09/04
       p = g_strdup_printf(_("Cannot open file '%s': unknown file type"), 
@@ -162,10 +165,84 @@ start_show(void)
       g_free(p);
       return;
   }
-  system(bff);
+
+  /* The following patch was received from Mamoru Tasaka, 2008/05/22
+     to solve potential security problems with the system() call */
+
+  if (pipe(pfd) == -1) {
+    fprintf(stderr, "Creating pipe failed\n");
+    return;
+  }
+
+  pid_i = fork(); 
+  if (pid_i == -1) {
+    fprintf(stderr, "Forking failed\n");
+    return;
+  }
+  else if (pid_i == 0) { /* child 1: do unzip or unrar */
+    close(pfd[0]);
+    close(1); /* close stdout*/
+    if (dup(pfd[1]) != 1 ) {
+      fprintf(stderr, "Dup failure\n");
+      return;
+    }
+    close(pfd[1]);
+    switch(pref.booktype) {
+      case ZIP_FILE:
+        if (debug) fprintf(stderr, "ZIP command: unzip -l %s\n", 
+		pref.lastbook);
+        execlp("unzip", "unzip", "-l", pref.lastbook, (char *)NULL);
+        return; /* should not reach here */
+      case RAR_FILE:
+        if (debug) fprintf(stderr, "RAR command: unrar -v %s\n", 
+		pref.lastbook);
+        execlp("unrar", "unrar", "v", pref.lastbook, (char *)NULL);
+        return; /* should not reach here */
+    }
+  }
+  else {
+    pid_j = fork() ; 
+    if (pid_j == -1) {
+      fprintf(stderr, "Forking failed\n");
+      return;
+    }
+    else if (pid_j == 0) { /* child 2; do grep */
+      close(pfd[1]);
+      close(0); /* close stdin */
+      if (dup(pfd[0]) != 0) {
+        fprintf(stderr, "Dup failure\n");
+        return;
+      }
+      close(pfd[0]);
+      if ((tmpf_fd = open(tmpf, O_WRONLY|O_TRUNC, S_IRUSR|S_IWUSR)) == -1) {
+        p = g_strdup_printf(_("Cannot open file '%s'"), tmpf);
+        ok_dialog(_("File error"), p);
+        g_free(p);
+        return;
+      }
+      close(1); /* close stdout */
+      if (dup(tmpf_fd) != 1) {
+        fprintf(stderr, "Dup failure\n");
+        return;
+      }
+      close(tmpf_fd);
+      execlp("grep", "grep", all_extensions, (char *)NULL);
+      return; /* should not reach here */
+    }
+    else { /* parent */
+      close(pfd[0]);
+      close(pfd[1]);
+      /* wait children */
+      waitpid(pid_i, 0, 0);
+      waitpid(pid_j, 0, 0);
+    }
+  }
+  bff = NULL;
+
+  /* End of this part of Mamoru Tasaka's patch */
 
   if (!g_file_test(tmpf, G_FILE_TEST_EXISTS)) {
-    printf(_("Cannot open temporary file %s\n"), tmpfile);
+    printf(_("Cannot open temporary file %s\n"), tmpf);
     g_free(bff);
     return;
   }
@@ -214,7 +291,6 @@ scroll_to(double where)
   GtkWidget *wdg;
   GtkAdjustment *adj;
 
-  if (debug) printf("[scroll] to %.2f\n");
   wdg = lookup_widget(MainWindow, "main_scroller");
   adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(wdg));
   if (debug)
@@ -229,13 +305,14 @@ scroll_to(double where)
 void
 request_page(int nr, GdkPixbuf **pxm, double *w, double *h)
 {
-  char *p, *bff, *esc;
-  int len, i, idx = 0;
+  char *p, *esc;
+  int len, i, idx = 0, pid;
   
   p = (char *)g_list_nth_data(pagelist, nr);
   len = strlen(p);
   esc = g_malloc(2*len + 1);
 
+  // Escaping characters which might be misinterpreted by the shell
   for (i = 0; i < len;  i++) {
     switch (p[i]) {
       case '[': case ']':
@@ -247,24 +324,48 @@ request_page(int nr, GdkPixbuf **pxm, double *w, double *h)
   }
   esc[idx] = '\0';
 
-  printf(_("Requesting page %d/%d (%s)\n"), nr+1, g_list_length(pagelist), esc);
+  printf(_("Requesting page %d/%d (%s)\n"), nr+1, 
+		g_list_length(pagelist), p);
 
-  switch (pref.booktype) {
-    case RAR_FILE:
-      bff = g_strdup_printf("unrar p -ierr -clr -- \"%s\" \"%s\" > %s",
-		pref.lastbook,
-		p,
-		tmpf);
-      break;
-    case ZIP_FILE:
-      bff = g_strdup_printf("unzip -p -C \"%s\" \"%s\" > %s",
-                pref.lastbook,
-                p,
-                tmpf);
-      break;
+  /* The following patch was received from Mamoru Tasaka, 2008/05/22
+     to solve potential security problems with the system() call */
+
+  pid = fork();
+  switch (pid) {
+    case -1:
+      fprintf(stderr, "Forking failed\n");
+      return;
+    case 0: /* child */
+      if ((tmpf_fd = open(tmpf, O_WRONLY|O_TRUNC, S_IRUSR|S_IWUSR)) == -1) {
+        p = g_strdup_printf(_("Cannot open file '%s'"), tmpf);
+        ok_dialog(_("File error"), p);
+        g_free(p);
+        return;
+      }
+      close(1); /* close stdout */
+      if (dup(tmpf_fd) != 1) {
+        fprintf(stderr, "Dup failure\n");
+        return;
+      }
+      close(tmpf_fd);
+      switch(pref.booktype) {
+        case RAR_FILE:
+          execlp("unrar",
+			"unrar", "p", "-ierr", "-clr", "--",
+			pref.lastbook, p, (char *)NULL);
+          return; /* should not reach here */
+        case ZIP_FILE:
+          /* for zip file, esc must be passed instead of p */
+          execlp("unzip",
+			"unzip", "-p", "-C",
+			pref.lastbook, esc, (char *)NULL);
+          return; /* should not reach here */
+      }
+      return; /* should not reach here */
+    default: /* parent */
+      waitpid(pid, 0, 0);
   }
-  system(bff);
-  g_free(bff);
+
   g_free(esc);
   
   *pxm = gdk_pixbuf_new_from_file(tmpf, NULL);
@@ -402,6 +503,60 @@ goto_end(void)
 }
 
 
+void
+file_list_expand(gboolean expand)
+{
+  GtkWidget *nav = lookup_widget(nav_window, "nav_toolbar"),
+    *paned;
+  GtkTreeView *tview;
+
+  if (expand) {
+    tview = GTK_TREE_VIEW(gtk_tree_view_new());
+    g_object_ref(G_OBJECT(nav));
+    gtk_container_remove(GTK_CONTAINER(nav_window), nav);
+
+    if (!pref.nav_hor) {
+      paned = gtk_hpaned_new();
+      gtk_widget_show(paned);
+      gtk_container_add(GTK_CONTAINER(nav_window), paned);
+      switch (pref.nav_pos) {
+        case NAV_NW:
+        case NAV_SW:
+	  gtk_paned_pack1(GTK_PANED(paned), nav, TRUE, FALSE);
+	  gtk_paned_pack2(GTK_PANED(paned), GTK_WIDGET(tview), TRUE, FALSE);
+          break;
+        case NAV_NE:
+        case NAV_SE:
+	  gtk_paned_pack2(GTK_PANED(paned), nav, TRUE, FALSE);
+	  gtk_paned_pack1(GTK_PANED(paned), GTK_WIDGET(tview), TRUE, FALSE);
+          break;
+      }
+    } else {
+      paned = gtk_vpaned_new();
+      switch (pref.nav_pos) {
+        case NAV_NW:
+        case NAV_NE:
+        case NAV_SW:
+        case NAV_SE:
+	  break;
+      }
+    }
+    g_object_unref(G_OBJECT(nav));
+    gtk_widget_show(GTK_WIDGET(tview));
+  } else {
+  }
+  file_list_expanded = expand;
+  printf("Expand %d\n", expand);
+}
+
+
+void
+file_list_tbtn_clicked(void)
+{
+  file_list_expand(!file_list_expanded);
+}
+
+
 void
 update_nav_position(void)
 {
@@ -493,6 +648,11 @@ clean_up(void)
 
 void
 main_canvas_keypressed(GdkEventKey *event)
+{
+}
+
+void
+main_canvas_keypressed1(GdkEventKey *event)
 {
   GtkWidget *w;
   char *wdg;
